#include "pch.h"

#include "JSONedit_jsoncons.h"
#include <jsoncons/json.hpp>
#include <jsoncons_ext/jsonpath/jsonpath.hpp>
#include <jsoncons_ext/jmespath/jmespath.hpp>
#include <jsoncons_ext/json_merge_patch/json_merge_patch.hpp>
#include <jsoncons_ext/jsonpatch/jsonpatch.hpp>
#include <assert.h>


using namespace jsoncons;


__declspec(dllexport) void JE_JSONCONS_CALL_CONV JE_jsoncons_get_interface_version(struct JE_JSONCONS_DLL_INTERFACE* version)
{
	assert(version);
	if (!version)
		return;
	version->majorVersion = JE_JSONCONS_DLL_INTERFACE_MAJOR_VERSION;
	version->minorVersion = JE_JSONCONS_DLL_INTERFACE_MINOR_VERSION;
}

static std::string retVal;
static std::string retExceptionText;
static std::string emptyStr = "";

__declspec(dllexport) int JE_JSONCONS_CALL_CONV JE_jsoncons_jsonpath_query(const char* jsonText, const char* path, const char** ret, const char** exception_text)
{
	*ret = emptyStr.c_str();
	*exception_text = emptyStr.c_str();

	try
	{
		// Parse the string of data into a json value
		ojson j = ojson::parse(jsonText);

		ojson result = jsonpath::json_query(j, path);
		retVal = "";
		result.dump_pretty(retVal);
		*ret = retVal.c_str();
		return 0;
	}
	catch (const std::exception& e)
	{
		retExceptionText = e.what();
		*exception_text = retExceptionText.c_str();
		return -1;
	}
	return -2;
}

__declspec(dllexport) int JE_jsoncons_jmespath_search(const char* jsonText, const char* expr, const char** ret, const char** exception_text)
{
	*ret = emptyStr.c_str();
	*exception_text = emptyStr.c_str();

	try
	{
		ojson j = ojson::parse(jsonText);
		ojson result = jsoncons::jmespath::search(j, expr);
		retVal = "";
		result.dump_pretty(retVal);
		*ret = retVal.c_str();
		return 0;
	}
	catch (const std::exception& e)
	{
		retExceptionText = e.what();
		*exception_text = retExceptionText.c_str();
		return -1;
	}
	return -2;
}

__declspec(dllexport) int JE_jsoncons_merge_patch_apply(const char* jsonText, const char* jsonPatch, const char** ret, const char** exception_text)
{
	*ret = emptyStr.c_str();
	*exception_text = emptyStr.c_str();
	try
	{
		json j = json::parse(jsonText);
		json patch = json::parse(jsonPatch);
		jsoncons::json_merge_patch::apply_merge_patch(j, patch);
		retVal = "";
		j.dump_pretty(retVal);
		*ret = retVal.c_str();
		return 0;
	}
	catch (const std::exception& e)
	{
		retExceptionText = e.what();
		*exception_text = retExceptionText.c_str();
		return -1;
	}
	return -2;
}

__declspec(dllexport) int JE_jsoncons_merge_patch_create(const char* jsonSource, const char* jsonTarget, const char** ret, const char** exception_text)
{
	*ret = emptyStr.c_str();
	*exception_text = emptyStr.c_str();
	try
	{
		ojson source = ojson::parse(jsonSource);
		ojson target = ojson::parse(jsonTarget);
		ojson patch = jsoncons::json_merge_patch::from_diff(source, target);
		retVal = "";
		patch.dump_pretty(retVal);
		*ret = retVal.c_str();
		return 0;
	}
	catch (const std::exception& e)
	{
		retExceptionText = e.what();
		*exception_text = retExceptionText.c_str();
		return -1;
	}
	return -2;
}

__declspec(dllexport) int JE_jsoncons_patch_apply(const char* jsonText, const char* jsonPatch, const char** ret, const char** exception_text)
{
	*ret = emptyStr.c_str();
	*exception_text = emptyStr.c_str();
	try
	{
		json j = json::parse(jsonText);
		json patch = json::parse(jsonPatch);
		jsoncons::jsonpatch::apply_patch(j, patch);
		retVal = "";
		j.dump_pretty(retVal);
		*ret = retVal.c_str();
		return 0;
	}
	catch (const std::exception& e)
	{
		retExceptionText = e.what();
		*exception_text = retExceptionText.c_str();
		return -1;
	}
	return -2;
}

__declspec(dllexport) int JE_jsoncons_patch_create(const char* jsonSource, const char* jsonTarget, const char** ret, const char** exception_text)
{
	*ret = emptyStr.c_str();
	*exception_text = emptyStr.c_str();
	try
	{
		ojson source = ojson::parse(jsonSource);
		ojson target = ojson::parse(jsonTarget);
		ojson patch = jsoncons::jsonpatch::from_diff(source, target);
		retVal = "";
		patch.dump_pretty(retVal);
		*ret = retVal.c_str();
		return 0;
	}
	catch (const std::exception& e)
	{
		retExceptionText = e.what();
		*exception_text = retExceptionText.c_str();
		return -1;
	}
	return -2;
}

__declspec(dllexport) void JE_JSONCONS_CALL_CONV JE_jsoncons_release(void)
{
	retVal.clear();
}